#
# 89776
# Kevin Kepros 06/01/01
# Only delete MANIFEST.MF file rather than complete META-INF directory.
#
# 88378
# Thien Pham 10/31/00
# remove all references iioptools.jar which was obsolete in release 3.5 
#
# 82739
# Amber Roy-Chowdhury 6/26/00
# Put the stub sources back; needed by internal build
#
# 74925
# Amber Roy-Chowdhury 2/25/00
# Break up input for awk to smaller bits
#
# Takes a client jar file, extracts the remote interfaces out of it, 
# runs rmic and wlmsg on the remore interfaces, and repackages the jar file.

#Changes made for robustness:
#1. Added exit status checks after every command execution of critical comands
#2. Added cleanup code for removal of resources created during execution
#3. User Inputs for interface names are validated by checking the existence of 
#   of the interface class file in the specified jar file
#4. At the end, a sanity check is made on the newly created jar file to
#   check if the stub & baseStub.[java|class] files exist and are of 
#   non-zero sizes
#5. Added a signal handler to trap and cleanup during interrupts

#The following variables are expected to be plugged in
#This script prereqs these variables 

#WAS_HOME=$(WAS_HOME) 
#export WAS_HOME
#
#JAVA_HOME=$(JAVA_HOME)  
#export JAVA_HOME

#If these variables are not defined then this script will rely on the 
#environment variables PATH, CLASSPATH, LD_LIBRARY_PATH/LIBPATH to be
#properly set. The script will not exit if these variables are not set

debug=
ifCount=0
interfaces=
verbose=
vverbose=
jarfile=
keep=
terminate=
reuseManifest=
# DOS style pwd
pwd=pwd 

printUsage () 
{ 
    if test -n "$debug" ; then  echo DEBUG entering printUsage ; fi
    echo Usage: wlmjar \<options\> 
    echo Options:
    echo     -J \<jar file\> \(required\)
    echo     -IF \<remote interface\> \(\<bean\> or \<bean\>Home, at least one required\)
    echo     -P \<package\> \(required\)
    echo     -M \(optional: reuse manifest\)
    echo     -V verbose mode
}

processArg ()
{
    if test -n "$debug" ; then echo DEBUG entering processArg ; fi
    extraShift= 
    case $1 in 
    -P|-p)            package=$2 ; extraShift=1 ;;
    -IF|-if|-If|-iF)  interfaces="${interfaces} $2" 
                      ifCount=`expr $ifCount + 1`; extraShift=1 ;;
    -J|-j)            jarfile=$2 ; extraShift=1 ;;
    -V|-v)            verbose=1 ;;
    -VV|-Vv|-vV|-vv)  vverbose=1 ; verbose=1 ;;
    -DD|-Dd|-dD|-dd)  verbose=1 ; debug=1 ;;
    -EE|-Ee|-eE|-ee)  echo on ;;
    -KK|-Kk|-kK|-kk)  keep=1 ;;
    -M|-m)            reuseManifest=1 ;;
    -MKS | -MKs |-MkS | -Mks | -mKS | -mKs | -mkS | -mks)  pwd=pwd ;; 
                      #MKS style passwd
    *)                echo "ERROR: Unrecognized option: $1 " 
                      printUsage
                      terminate=1
    esac
}

processInterface ()
{
if test -n "$debug" ; then echo DEBUG entering processInterface ; fi
if test -n "$debug" ; then echo "DEBUG params $1, $2" ; fi

tmp=$2
tmp=`echo $tmp | /usr/bin/sed 's/\./\//g'`
ifDir=${topDir}/${workingDir}/$tmp
if test -n "$debug" ; then echo "DEBUG ifDir=$ifDir" ; fi
cd $ifDir
if test $? -ne 0
then
echo Error: Could not change to directory  $ifDir
terminate=1
return 1
fi
if test -n "$debug" ; then pwd ; fi
if test ! -r "${ifDir}/${1}.class" 
then
if test ! -r "${1}.java"
then
echo Error: "Could not find ${ifDir}/${1}.class, ${1}.java in $jarfile "
terminate=1
return 1
fi

javacOpts="-classpath $classpath"
if test -n "$verbose" ; then echo "javac $javacOpts ${1}.java" ; fi
javac $javacOpts ${1}.java
if test $? -ne 0
then
echo "Error: Cannot compile  ${1}.java"
terminate=1
return 1
fi
fi

# work around rmic bug
# if the stubs exist already, rmic will not attempt to recreate them
# this causes problems for us as we need the stub sources 
# so we need to force rmic to run with 
# -keepgenerated (NOTE: same as -keep in NT)
# so we remove the stubs if they are present

stubClass="_${1}_Stub.class"
if test -r "$stubClass"  
then  
if test -n "$verbose" ; then echo "removing old stub $stubClass" ; fi
rm $stubClass
if test $? -ne 0
then
echo Error: Failed to remove leftover Stub class $stubClass
echo Error: if the stubs exist already, rmic will not attempt to recreate them
terminate=1
return
fi
fi

rmicOpts="-classpath $classpath -keepgenerated -iiop -g -d ${topDir}/${workingDir}"

if test -n "$vverbose" ; then rmicOpts="$rmicOpts -verbose" ; fi
if test `uname` = "HP-UX"; then rmicOpts="-classic $rmicOpts" ; fi
if test -n "$verbose" ; then echo "rmic $rmicOpts $2.$1" ; fi


#SETTING CLASSPATH IN THE ENVIRONMENT FOR rmic: just to make sure...
OLD_CLASSPATH=$CLASSPATH
CLASSPATH=$classpath; export CLASSPATH

rmic $rmicOpts $2.$1
if test $? -ne 0
then
echo Error: Failed to run rmic on  $2.$1
echo Please check that you are using the iiop version of rmic in your PATH
echo and your LD_LIBRARY_PATH/LIBPATH environment variables
CLASSPATH=$OLD_CLASSPATH ; export CLASSPATH
terminate=1
return 1
fi

#RESETTING CLASSPATH
CLASSPATH=$OLD_CLASSPATH ; export CLASSPATH

localClasspath=$classpath:${topDir}/${workingDir}
javaOpts="-classpath $localClasspath"
wlmsgOpts="-p $2 -name $1"
if test -n "$verbose" 
then 
echo "java $javaOpts com.ibm.ejs.wlm.wlmsg.wlmsg $wlmsgOpts"
fi
java $javaOpts com.ibm.ejs.wlm.wlmsg.wlmsg $wlmsgOpts
if test $? -ne 0
then
echo Error: Failed to run com.ibm.ejs.wlm.wlmsg.wlmsg
echo check classpath
terminate=1
return 1
fi
javacOpts="-classpath $localClasspath -d ${topDir}/${workingDir}"
if test -n "$verbose"; then  echo "javac $javacOpts _${1}_BaseStub.java" ; fi
javac $javacOpts _${1}_BaseStub.java
if test $? -ne 0
then
echo "Error: Failed to compile  _${1}_BaseStub.java"
terminate=1
return 1
fi

# D79206 Delete the stub source file after compilation so it won't end up in jar file
# D82739 put the sources back for now; needed by our internal build
#if test -n "$verbose"; then  echo "rm -f _${1}_BaseStub.java" ; fi
#rm -f _${1}_BaseStub.java

if test -n "$verbose"; then  echo "javac $javacOpts _${1}_Stub.java" ; fi
javac $javacOpts _${1}_Stub.java
if test $? -ne 0
then
echo "Error: Failed to compile  _${1}_Stub.java"
terminate=1
return 1
fi

# D79206 Delete the stub source file after compilation so it won't end up in jar file
# D82739 put the sources back for now; needed by our internal build
#if test -n "$verbose"; then  echo "rm -f _${1}_Stub.java" ; fi
#rm -f _${1}_Stub.java

if test -n "$debug" ; then $pwd ; fi
if test -n "$debug" ; then echo DEBUG exiting processInterface ; fi
}
cleanupandexit ()
{

if test -n "$keep"
then
exit 1
fi

echo "Error in wlmjar file creation..."
echo "Cleaning up ..."


if test -f "${topDir:=`pwd`}/_wlm_${jarfile:="Unknown"}" 
then 
echo "Removing wlmized jar file: ${topDir}/_wlm_${jarfile}"
rm ${topDir}/_wlm_${jarfile} 
fi

cd ${topDir}

if test -d "${topDir}/${workingDir:="_wlmjar"}"
then
echo "Removing working directory: ${topDir}/${workingDir}"
rm -r -f ${topDir}/${workingDir}
fi

exit 1;
}

if test $# -eq 0 
then
   echo ERROR: Invalid command line.
   printUsage
   exit 1
fi
#Add signal handler to trap interupts for cleanupandexit
trap 'cleanupandexit'  1 2 3 9 15
trap 'cd ${topDir:=`pwd`}; if test -d "${topDir}/${workingDir:="_wlmjar"}" ; then if test -z "$keep" ; then rm -r -f ${topDir}/${workingDir} ;  fi ; fi' 0

# process arguments

#Source setupCmdLine.sh 
setupCmdLineDir=`echo $0 | sed s/wlmjar.sh//`
if test -f ${setupCmdLineDir}setupCmdLine.sh ; then . ${setupCmdLineDir}setupCmdLine.sh
fi

while test $# -gt 0
do
    processArg $1 $2
    if test -n "$terminate" ; then exit 1 ; fi
    shift
    if test -n "$extraShift" ; then shift ; fi
done

if test -n "$verbose" ; then  echo ". ${setupCmdLineDir}setupCmdLine.sh" ; fi

#check that jar file, package, and at least one remote interface were specified
if test -z "$jarfile"
then
    echo ERROR: jar file was not specified
    printUsage
    exit 1
fi
if test -z "$package" 
then
    echo ERROR: Package was not specified
    printUsage
    exit 1    
fi
if test -z "$interfaces" 
then
    echo ERROR: at least one remote interface must be specified
    printUsage
    exit 1
fi
if test ! -r "$jarfile" 
then
    echo "ERROR: jar file $jarfile not found or is not readable"
    exit 1
fi
#check that package and remote interfaces are valid
#awk is breaking if a large number of interfaces is given: string too long
#hence breaking up the set of interfaces into sets of 9 interfaces at a time
set $interfaces
while test $# -gt 0
do
if test $# -lt 2 
then 
 smallInterfaces=$*
 shift $# 
else
 smallInterfaces="$1 $2"
 shift 2
fi

jar tvf $jarfile 2>&1 | /usr/bin/awk 'BEGIN { 
  n=split("'"$smallInterfaces"'",IF)
  packageDir="'"`echo $package | /usr/bin/sed 's/\./\//g'`"'"
  for(i=1;i<=n;i++)
  {
    interfaceClassFile[i]=packageDir "/" IF[i] ".class"
    c[i]=0
  }
}
{
  for ( i=1 ; i<=n ; i++)
   if(index($0,interfaceClassFile[i]) > 0 && $1 > 0)
    c[i]=1
   

}
END {
   ret=1
   for(i=1;i<=n;i++)
    ret=ret*c[i]
   exit  ret
}'


if test $? -eq 0
then
echo "Error: 1 or more of the specified interfaces and/or package"
echo "Package: $package and Interfaces: $interfaces"
echo "do not exist in the jarfile $jarfile"
cleanupandexit
fi

done

if test -n "$verbose"
then 
echo "jarfile=$jarfile"
if test -n "$package" ; then  echo "package=$package" ; fi
echo "remote interfaces=$interfaces"
echo "CLASSPATH=$CLASSPATH"
echo "PATH=$PATH"
fi


#startProcessing
# create a working directory
topDir=`$pwd`
workingDir=_wlmjar
if test -d "$workingDir" 
then
echo "Directory $workingDir exists already: it will be removed and recreated." 
echo Hit CTRL-C now to terminate or press enter to continue
junk=
read junk
rm -r -f $workingDir 
fi
if test -n "$verbose" ; then echo creating working directory ... ; fi
mkdir $workingDir
if test $? -ne 0 
then
echo "Error creating working directory: ${workingDir}"
cleanupandexit
fi

#extract jar file into directory
jaroptions=xf
if test -n "$vverbose" ; then jaroptions="v$jaroptions" ; fi
cp $jarfile $workingDir
if test $? -ne 0 
then
echo "Error in copying jarfile: ${jarfile} to working directory: ${workingDir}"
cleanupandexit
fi
cd ${topDir}/${workingDir}
if test $? -ne 0 
then
echo "Error in changing to working directory: ${workingDir}"
cleanupandexit
fi
if test -n "$verbose" 
then
echo extracting jarfile ...
echo "jar $jaroptions $jarfile"
fi
jar $jaroptions $jarfile
if test $? -ne 0 
then
echo "Error in extracting from jarfile: ${jarfile} in working directory: ${workingDir} "
cleanupandexit
fi
rm $jarfile

if test -z "$WAS_HOME" -o -z "$JAVA_HOME" 
then
    echo ------------------------------------------------
    echo WARNING: WAS_HOME and/or JAVA_HOME not set, 
    echo ------------------------------------------------
    echo Using environment CLASSPATH to locate jar files: 
    echo iioprt.jar, ujc.jar, classes.zip 
    echo ------------------------------------------------
    echo Environment CLASSPATH is $CLASSPATH 
    echo ------------------------------------------------
    echo Using environment PATH to locate iiop rmic
    echo ------------------------------------------------
    echo Environment PATH is $PATH
    echo ------------------------------------------------
    echo Using LD_LIBRARY_PATH/LIBPATH to locate libioser.a
    echo ------------------------------------------------
    echo Environment LD_LIBRARY_PATH is $LD_LIBRARY_PATH
    echo Environment LIBPATH is $LIBPATH
    echo ------------------------------------------------
    classpath=${CLASSPATH:+${CLASSPATH}:}.:${topDir}/${jarfile}:${topDir}/${workingDir}${JAVA_HOME:+:${JAVA_HOME}/jre/lib/rt.jar}

else

    classpath=${CLASSPATH:+${CLASSPATH}:}${WAS_HOME}/lib/ujc.jar:${WAS_HOME}/lib/iioprt.jar:.:${topDir}/${jarfile}:${topDir}/${workingDir}${JAVA_HOME:+:${JAVA_HOME}/jre/lib/rt.jar}
    PATH=${WAS_HOME}/bin${PATH:+:${PATH}}
    export PATH
    LD_LIBRARY_PATH=${WAS_HOME}/bin${LD_LIBRARY_PATH:+:${LD_LIBRARY_PATH}}
    export LD_LIBRARY_PATH
    LIBPATH=${WAS_HOME}/bin${LIBPATH:+:${LIBPATH}}
    export LIBPATH
    if test -n "$verbose" 
    then
     echo ------------------------------------------------
     echo Environment LD_LIBRARY_PATH=$LD_LIBRARY_PATH
     echo ------------------------------------------------
     echo Environment LIBPATH is $LIBPATH
     echo ------------------------------------------------
     echo Environment PATH is $PATH
     echo ------------------------------------------------
   fi
fi

# process the interfaces one by one
if test -n "$verbose" ; then  echo processing interfaces ... ; fi
for i in $interfaces 
do
processInterface $i $package
if test -n "$terminate" ; then cleanupandexit ; fi
done
if test -n "$terminate" ; then cleanupandexit ; fi 

#create new jarfile

cd ${topDir}/${workingDir}
if test -n "$reuseManifest"  
then 
cp META-INF/MANIFEST.MF ${topDir}/_wlm_manifest
if test $? -ne 0 
then
echo "Error in copying manifest file  META-INF/MANIFEST.MF to ${topDir}/_wlm_manifest"
echo "cannot resuse Manifest"
reuseManifest=
fi
fi
rm -f META-INF/MANIFEST.MF
newjarfile=_wlm_$jarfile
jaroptions=cf
if test -n "$vverbose" ; then  jaroptions="v$jaroptions" ; fi
if test -n "$reuseManifest" 
then 
jaroptions="m$jaroptions ${topDir}/_wlm_manifest" 
fi
if test -n "$verbose"
then
echo constructing jarfile ...
echo "jar $jaroptions ../$newjarfile *"
fi
jar $jaroptions ../$newjarfile *
if test $? -ne 0 
then
echo "Error in creating new jarfile: ${jarfile} in directory: `pwd`/.. "
cleanupandexit
fi

cd $topDir
#Check to see if the the new jar file created has the 
#wlmized stub classes and sources and their file sizes are not zero
if test -n "$verbose" 
then 
echo checking the new jarfile: $newjarfile for the following. 
echo "1. <interface>_Stub.class exists"
echo "2. <interface>_BaseStub.class exists"
echo "3. The file sizes are not zero"
fi

set $interfaces
while test $# -gt 0
do
if test $# -lt 2 
then 
 smallInterfaces=$* 
 shift $#
else
 smallInterfaces="$1 $2"
 shift 2
fi

# D79206 - The tests for the java files have been removed since we
#          no longer put the Stub source files in the jar file.
jar tvf $newjarfile 2>&1 | awk 'BEGIN { 
  n=split("'"$smallInterfaces"'",IF)
  packageDir="'"`echo $package | /usr/bin/sed 's/\./\//g'`"'"
  for(i=1;i<=n;i++)
  {
    wlmedStubClassFile[i]=packageDir "/_" IF[i] "_Stub.class"
    wlmedBaseStubClassFile[i]=packageDir "/_" IF[i] "_BaseStub.class"
    s1[i]=0
    s2[i]=0
  }
}
{
  for ( i=1 ; i<=n ; i++)
  {
   if(index($0,wlmedStubClassFile[i]) > 0 && $1 > 0)
    s1[i]=1
   else if(index($0,wlmedBaseStubClassFile[i]) > 0 && $1 > 0)
    s2[i]=1
   }
}
END {
   ret=1
   for(i=1;i<=n;i++)
    ret=ret*s1[i]*s2[i]
   exit  ret
}'

if test $? -eq 0
then
echo "Error in creation of new Jar File:$newjarfile"
cleanupandexit
fi
done
#EVERYTHING SEEMS TO BE OK, cleanup only working directory and exit
if test -n "$verbose" ; then echo new jarfile check succesful... ; fi

# remove working directory
if test -n "$keep" ; then exit 0 ; fi

if test -n "$verbose" ; then echo removing working directory ... ; fi
rm -r -f $workingDir
if test -n "$reuseManifest" ; then rm -f _wlm_manifest ; fi 

exit 0
